@@ -42,11 +42,12 @@ def makeLogfile(fromLoc, toLocPath, logDir=None):
         return 
     destDir = os.path.join(ARGS.hd_path, toLocPath)
     if not logDir:
+        fromLocDir = fromLoc
         if os.path.isfile(fromLoc):
-            fromLoc = os.path.dirname(fromLoc)
-            if re.search(r'\bS\d+$', fromLoc):
-                fromLoc = os.path.dirname(fromLoc)
-        originName, ext = os.path.splitext(os.path.basename(fromLoc))
+            fromLocDir = os.path.dirname(fromLoc)
+            if re.search(r'\bS\d+$', fromLocDir):
+                fromLocDir = os.path.dirname(fromLocDir)
+        originName, ext = os.path.splitext(os.path.basename(fromLocDir))
     else:
         originName = os.path.basename(logDir)
     if not originName:
@@ -264,11 +265,11 @@ def isMediaFileType(file_ext):
 
 
 def copyTVSeasonItems(tvSourceFullPath, tvFolder, seasonFolder, groupName,
-                      resolution):
+                      resolution, folderTmdbParser=None):
     if os.path.isdir(os.path.join(tvSourceFullPath, 'BDMV')):
         # break, process BDMV dir for this dir
         bdmvTVFolder = os.path.join(tvFolder, seasonFolder)
-        processBDMV(tvSourceFullPath, bdmvTVFolder, CATNAME_TV)
+        processBDMV(tvSourceFullPath, bdmvTVFolder, CATNAME_TV, tmdbParser=folderTmdbParser)
         return
 
     # catutil = TorCategory()
@@ -313,21 +314,27 @@ def genTVSeasonEpisonGroup(mediaFilename, groupName, resolution):
     tt = TorTitle(mediaFilename)
     tvTitle, tvYear, tvSeason, tvEpisode, cntitle = tt.title, tt.yearstr, tt.season, tt.episode, tt.cntitle
     # tvTitle = fixNtName(tvTitle)
+    cutName = cutOriginName(mediaFilename)
 
     tvEpisode = re.sub(r'^Ep\s*', 'E', tvEpisode, flags=re.I)
-    filename, file_ext = os.path.splitext(mediaFilename)
-    ch1 = '- ' if (resolution or groupName) else ''
-    ch2 = '_' if (resolution and groupName) else ''
-
-    tvname = '%s %s %s%s %s%s%s' % (tvTitle,
+    # filename, file_ext = os.path.splitext(mediaFilename)
+    # ch1 = '- ' if (resolution or groupName) else ''
+    # ch2 = '_' if (resolution and groupName) else ''
+    # tvname = '%s %s %s%s %s%s%s' % (tvTitle,
+    #                                 ('(' + tvYear + ')') if tvYear else '',
+    #                                 tvSeason.upper() if tvSeason else '',
+    #                                 tvEpisode.upper() if tvEpisode else '',
+    #                                 (tt.subEpisode+' ') if tt.subEpisode else '',
+    #                                 ch1+resolution if resolution else '',
+    #                                 ch2+groupName if groupName else '')
+    tvname = '%s %s %s%s %s - %s' % (tvTitle,
                                     ('(' + tvYear + ')') if tvYear else '',
                                     tvSeason.upper() if tvSeason else '',
                                     tvEpisode.upper() if tvEpisode else '',
                                     (tt.subEpisode+' ') if tt.subEpisode else '',
-                                    ch1+resolution if resolution else '',
-                                    ch2+groupName if groupName else '')
+                                    cutName)
 
-    tvname = tvname.strip() + file_ext
+    tvname = tvname.strip()
 
     # filename, file_ext = os.path.splitext(mediaFilename)
     # ch1 = ' - ' if (resolution or groupName) else ''
@@ -341,13 +348,16 @@ def genTVSeasonEpisonGroup(mediaFilename, groupName, resolution):
 
 
 def countMediaFile(filePath):
-    types = ('*.mkv', '*.mp4', '*.ts')
+    types = ('*.mkv', '*.mp4', '*.ts', '*.m2ts')
     curdir = os.getcwd()
-    os.chdir(filePath)
     mediaCount = 0
-    for files in types:
-        mediaCount += len(glob.glob(files))
-    os.chdir(curdir)
+    try:
+        os.chdir(filePath)
+        for files in types:
+            mediaCount += len(glob.glob(files))
+        os.chdir(curdir)
+    except:
+        pass
     return mediaCount
 
 
@@ -407,7 +417,7 @@ def copyTVFolderItems(tvSourceFolder, genFolder, folderSeason, groupName,
     if os.path.isdir(os.path.join(tvSourceFolder, 'BDMV')):
         if ARGS.full_bdmv or ARGS.extract_bdmv:
             # a BDMV dir in a TV folder, treat as Movie
-            processBDMV(tvSourceFolder, genFolder, 'MovieM2TS')
+            processBDMV(tvSourceFolder, genFolder, 'MovieM2TS', tmdbParser=folderTmdbParser)
             targetDirHook(os.path.join('MovieM2TS', genFolder), tmdbidstr=str(folderTmdbParser.tmdbid))
         else:
             print('\033[31mSkip BDMV/ISO  %s \033[0m' % genFolder)
@@ -419,7 +429,7 @@ def copyTVFolderItems(tvSourceFolder, genFolder, folderSeason, groupName,
     if not os.path.isdir(tvSourceFolder):
         return
 
-    for tvitem in os.listdir(tvSourceFolder):
+    for tvitem in sorted(os.listdir(tvSourceFolder)):
         if uselessFile(tvitem):
             print('\033[34mSKIP useless file: [%s]\033[0m ' % tvitem)
             continue
@@ -431,7 +441,7 @@ def copyTVFolderItems(tvSourceFolder, genFolder, folderSeason, groupName,
         if os.path.isdir(tvitemPath):
             seasonFolder = getSeasonFromFolderName(tvitem, failDir=parseSeason)
             copyTVSeasonItems(tvitemPath, genFolder, seasonFolder, parseGroup,
-                              resolution)
+                              resolution, folderTmdbParser=folderTmdbParser)
         else:
             filename, file_ext = os.path.splitext(tvitemPath)
             if isMediaFileType(file_ext):
@@ -449,8 +459,22 @@ def copyTVFolderItems(tvSourceFolder, genFolder, folderSeason, groupName,
     targetDirHook(os.path.join(CATNAME_TV, genFolder), tmdbidstr=str(folderTmdbParser.tmdbid))
 
 
+def cutOriginName(srcOriginName):
+    m1 = re.search( r'^.*\b(720p|1080[pi]|2160p|576i)[\. ]*', srcOriginName, flags=re.I)
+    sstr = srcOriginName
+    if m1:
+        sstr = srcOriginName[m1.span(1)[0]:]
+    else:
+        m2 = re.search( r'\b((19\d{2}\b|20\d{2})(-19\d{2}|-20\d{2})?)\b(?!.*\b\d{4}\b.*)', srcOriginName, flags=re.A | re.I)
+        if m2:
+            sstr = sstr[m2.span(1)[1]:]
+    sstr = re.sub(r'^[. ]*', '', sstr)
+    sstr = re.sub(r'-', '_', sstr)
+    return sstr
+
+
 def genMovieTMDbOriginName(mediaSrc, movieName, year, nameParser=None):
-    originName = os.path.basename(mediaSrc)
+    originName = cutOriginName(os.path.basename(mediaSrc))
     # filename, file_ext = os.path.splitext(mediaSrc)
     ch1 = ' - '
     tmdbTail = ''
@@ -524,12 +548,12 @@ def genCatFolderName(parser):
 
 
 def isCollections(folderName):
-    return re.search(r'(\bPack$|合集|Anthology|Trilogy|Quadrilogy|Tetralogy|(?<!Criterion)\s+Collections?|国语配音4K动画电影$)',
+    return re.search(r'(\bPack$|合集|Anthology|Trilogy|Quadrilogy|Tetralogy|(?<!Criterion[ .])Collections?|国语配音4K动画电影$)',
                      folderName,
                      flags=re.I)
 
 
-def processBDMV(mediaSrc, folderGenName, catFolder):
+def processBDMV(mediaSrc, folderGenName, catFolder, tmdbParser=None):
     destCatFolderName = os.path.join(catFolder, folderGenName)
     if ARGS.full_bdmv:
         for bdmvItem in os.listdir(mediaSrc):
@@ -544,11 +568,29 @@ def processBDMV(mediaSrc, folderGenName, catFolder):
                   mediaSrc)
             return
 
-        largestStreams = getLargestFiles(bdmvDir)
-        for stream in largestStreams:
-            # fn, ext = os.path.splitext(stream)
-            tsname = os.path.basename(mediaSrc) + ' - ' + os.path.basename( stream)
-            targetCopy(stream, destCatFolderName, tsname)
+        largestStreams = sorted(getLargestFiles(bdmvDir))
+        folderGenList = re.split(r'\/|\\', folderGenName)
+        folderName = folderGenList[0]
+        diskName = "" if len(folderGenList) <= 1 else folderGenList[1]
+
+        if tmdbParser and tmdbParser.tmdbcat == 'tv':
+            m = re.search(r"(S|Season)(\d+)", diskName, re.I)
+            if m:
+                ssName = "S%02d" % int(m[2])
+            else:
+                ssName = "S01"
+            destCatFolderName = os.path.join(catFolder, folderName, ssName)
+            time.sleep(1)
+            epCount = countMediaFile(os.path.join(ARGS.hd_path, destCatFolderName))
+            for epidx, stream in enumerate(largestStreams):
+                tsname = "%s %sE%02d %s_%s" % (folderName, ssName, epCount+epidx+1, diskName, os.path.basename(stream))
+                # tsname = '%s %s_%s' % (folderName, diskName, os.path.basename(stream))
+                targetCopy(stream, destCatFolderName, tsname)
+        else:
+            for stream in largestStreams:
+                # tsname = folderGenName + ' - ' + os.path.basename(mediaSrc) + '_' + os.path.basename( stream)
+                tsname = folderName + ' - ' + os.path.basename( stream)
+                targetCopy(stream, destCatFolderName, tsname)
 
     else:
         print('\033[31mSkip BDMV/ISO  %s \033[0m' % mediaSrc)
@@ -565,7 +607,7 @@ def processMovieDir(mediaSrc, folderCat, folderGenName, folderTmdbParser):
     if os.path.isdir(os.path.join(mediaSrc, 'BDMV')):
         # break, process BDMV dir for this dir
         if ARGS.full_bdmv or ARGS.extract_bdmv:
-            processBDMV(mediaSrc, folderGenName, 'MovieM2TS')
+            processBDMV(mediaSrc, folderGenName, 'MovieM2TS', tmdbParser=folderTmdbParser)
             targetDirHook(os.path.join('MovieM2TS', folderGenName), tmdbidstr=str(folderTmdbParser.tmdbid))
         else:
             print('\033[31mSkip BDMV/ISO  %s \033[0m' % mediaSrc)
@@ -591,6 +633,7 @@ def processMovieDir(mediaSrc, folderCat, folderGenName, folderTmdbParser):
         if (os.path.isdir(os.path.join(mediaSrc, movieItem))):
             # Dir in movie folder
             if os.path.isdir(os.path.join(mediaSrc, movieItem, 'BDMV')):
+                print(" Alert: MovieBDMV in a Movie dir.....?")
                 processBDMV(os.path.join(mediaSrc, movieItem),
                             os.path.join(folderGenName, movieItem),
                             'MovieM2TS')
@@ -690,7 +733,7 @@ def targetDirHook(targetDir, tmdbidstr=''):
     return
 
 
-def processOneDirItem(cpLocation, itemName, imdbidstr=''):
+def processOneDirItem(cpLocation, itemName, imdbidstr='', tmdbidstr=''):
     global CUR_MEDIA_NAME 
     CUR_MEDIA_NAME = itemName
     mediaSrc = os.path.join(cpLocation, itemName)
@@ -698,11 +741,11 @@ def processOneDirItem(cpLocation, itemName, imdbidstr=''):
         print('\033[31mSKIP symbolic link: [%s]\033[0m ' % mediaSrc)
         return
 
+    print(" >> [%s] %s %s" % (itemName, imdbidstr, tmdbidstr))
     cat = setArgsCategory()
     p = TMDbNameParser(ARGS.tmdb_api_key, ARGS.tmdb_lang, ccfcat_hard=cat)
-    p.parse(itemName, useTMDb=(ARGS.tmdb_api_key is not None), hasIMDbId=imdbidstr)
+    p.parse(itemName, useTMDb=(ARGS.tmdb_api_key is not None), hasIMDbId=imdbidstr, hasTMDbId=tmdbidstr)
     p.title = fixNtName(p.title)
-    
     cat = genCatFolderName(p)
 
     destFolderName = genMediaFolderName(p)
@@ -776,7 +819,7 @@ def processOneDirItem(cpLocation, itemName, imdbidstr=''):
                 targetCopy(mediaSrc, cat)
                 targetDirHook(os.path.join(cat, itemName), tmdbidstr='notfound')
 
-        elif cat in ['HDTV', 'Audio']:
+        elif cat in ['Audio']:
             targetCopy(mediaSrc, cat)
             targetDirHook(os.path.join(cat, itemName), tmdbidstr='audio')
         elif cat in ['eBook']:
@@ -903,6 +946,9 @@ def loadArgs():
                         default='',
                         help='call this script with destination folder path after link/move')
     parser.add_argument('--imdbid',
+                        default='',
+                        help='specify the IMDb id, -s single mode only')
+    parser.add_argument('--tmdbid',
                         default='',
                         help='specify the TMDb id, -s single mode only')
     parser.add_argument('--site-str',
@@ -991,10 +1037,11 @@ def main():
         searchCache.openCache()
 
     argIMDb = ARGS.imdbid if (ARGS.single and ARGS.imdbid) else ''
+    argTMDb = ARGS.tmdbid if (ARGS.single and ARGS.tmdbid) else ''
 
     if os.path.isfile(cpLocation):
         processOneDirItem(os.path.dirname(cpLocation),
-                          os.path.basename(os.path.normpath(cpLocation)), argIMDb)
+                          os.path.basename(os.path.normpath(cpLocation)), imdbidstr=argIMDb, tmdbidstr=argTMDb)
     else:
         if ARGS.single and not isCollections(cpLocation):
             # processOneDirItem(os.path.dirname(cpLocation),
@@ -1002,8 +1049,8 @@ def main():
             
             parentLocation, itemName, folderimdb = parseFolderIMDbId(os.path.dirname(cpLocation),
                               os.path.basename(os.path.normpath(cpLocation)))
-            if argIMDb:
-                processOneDirItem(parentLocation, itemName, argIMDb)
+            if argIMDb or argTMDb:
+                processOneDirItem(parentLocation, itemName, imdbidstr=argIMDb, tmdbidstr=argTMDb)
             else:
                 processOneDirItem(parentLocation, itemName, folderimdb)
 
@@ -1025,16 +1072,16 @@ def main():
                                 print('\033[32mSkipping. File previously linked: %s \033[0m' % (fn))
                                 continue
                             else:
-                                processOneDirItem(packDir, fn, imdbidstr='')
                                 searchCache.append(fn)
+                        processOneDirItem(packDir, fn, imdbidstr='')
                 else:
                     if ARGS.cache:
                         if searchCache.isCached(itemName):
                             print('\033[32mSkipping. File previously linked: %s \033[0m' % (itemName))
                             continue
                         else:
-                            processOneDirItem(parentLocation, itemName, folderimdb)
                             searchCache.append(itemName)
+                    processOneDirItem(parentLocation, itemName, folderimdb)
 
     if ARGS.cache:
         searchCache.closeCache()
